package com.example.nearby;

import android.app.Activity;
import android.content.Intent;
import android.content.IntentSender;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.KeyEvent;
import android.view.inputmethod.EditorInfo;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.CommonStatusCodes;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.ResultCallback;
import com.google.android.gms.common.api.Status;
import com.google.android.gms.nearby.Nearby;
import com.google.android.gms.nearby.messages.Message;
import com.google.android.gms.nearby.messages.MessageListener;
import com.google.android.gms.nearby.messages.PublishCallback;
import com.google.android.gms.nearby.messages.PublishOptions;
import com.google.android.gms.nearby.messages.Strategy;
import com.google.android.gms.nearby.messages.SubscribeCallback;
import com.google.android.gms.nearby.messages.SubscribeOptions;

public class MainActivity extends AppCompatActivity implements GoogleApiClient.ConnectionCallbacks, GoogleApiClient.OnConnectionFailedListener {

    private static final String TAG = MainActivity.class.getSimpleName();
    private GoogleApiClient mGoogleApiClient;
    private boolean mResolvingError;

    private MessageListener mMessageListener;

    private static final int REQUEST_RESOLVE_ERROR = 1001;

    private Message mMessage;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);


        final ListView listView = (ListView) findViewById(R.id.listView);

        final ArrayAdapter adapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1);
        listView.setAdapter(adapter);

        EditText editText = (EditText) findViewById(R.id.editText);

        editText.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
                boolean result = false;
                if (actionId == EditorInfo.IME_ACTION_DONE) {
                    String message = v.getText().toString();

                    adapter.add(message);
                    int itemCount = listView.getCount();
                    listView.setSelection(itemCount - 1);

                    publish(message);

                    v.setText("");
                    result = true;
                }
                return result;
            }
        });

        mMessageListener = new MessageListener() {
            @Override
            public void onFound(final Message message) {
                final String nearbyMessageString = new String(message.getContent());
                adapter.add(nearbyMessageString);

                int itemCount = listView.getCount();
                listView.setSelection(itemCount - 1);

                Log.i(TAG, "メッセージ : " + nearbyMessageString);
            }

            // Called when a message is no longer detectable nearby.
            @Override
            public void onLost(final Message message) {
                final String nearbyMessageString = new String(message.getContent());
                // Take appropriate action here (update UI, etc.)

                Log.i(TAG, "onLost : " + nearbyMessageString);
            }

        };

        mGoogleApiClient = new GoogleApiClient.Builder(this)
                .addApi(Nearby.MESSAGES_API)
                .addConnectionCallbacks(this)
                .addOnConnectionFailedListener(this)
                .build();
    }

    @Override
    protected void onStart() {
        super.onStart();

        mGoogleApiClient.connect();
    }

    @Override
    protected void onStop() {
        if (mGoogleApiClient.isConnected() && !isChangingConfigurations()) {
            unsubscribe();
            unpublish();

            mGoogleApiClient.disconnect();
        }

        super.onStop();
    }

    @Override
    public void onConnected(Bundle bundle) {
        subscribe();
    }

    @Override
    public void onConnectionSuspended(int i) {
        unsubscribe();
    }

    @Override
    public void onConnectionFailed(ConnectionResult connectionResult) {

    }

    private void subscribe() {

        if (!mGoogleApiClient.isConnected()) {
            if (!mGoogleApiClient.isConnecting()) {
                mGoogleApiClient.connect();
            }
        } else {
            Strategy strategy = new Strategy.Builder()
                    .setDiscoveryMode(Strategy.DISCOVERY_MODE_DEFAULT)
                    .setDistanceType(Strategy.DISTANCE_TYPE_DEFAULT)
                    .setTtlSeconds(Strategy.TTL_SECONDS_INFINITE)
                    .build();

            SubscribeOptions options = new SubscribeOptions.Builder()
                    .setStrategy(strategy)
                    .setCallback(new SubscribeCallback() {
                        @Override
                        public void onExpired() {
                            // タイムアウト
                            Log.i(TAG, "タイムアウト");
                        }
                    }).build();


            Nearby.Messages.subscribe(mGoogleApiClient, mMessageListener, options)
                    .setResultCallback(new ResultCallback<Status>() {
                        @Override
                        public void onResult(Status status) {
                            if (status.isSuccess()) {
                                // 購読成功
                                Log.i(TAG, "成功");
                            } else {
                                // 購読失敗
                                Log.i(TAG, "失敗");

                                handleUnsuccessfulNearbyResult(status);
                            }
                        }
                    });
        }
    }

    private void unsubscribe() {
        Nearby.Messages.unsubscribe(mGoogleApiClient, mMessageListener);
    }

    private void handleUnsuccessfulNearbyResult(Status status) {

        if (mResolvingError) {
            return;
        } else if (status.hasResolution()) {
            try {
                mResolvingError = true;
                status.startResolutionForResult(this,
                        REQUEST_RESOLVE_ERROR);
            } catch (IntentSender.SendIntentException e) {
                mResolvingError = false;
                Log.i(TAG, "エラー:", e);
            }
        } else {
            if (status.getStatusCode() == CommonStatusCodes.NETWORK_ERROR) {
                Toast.makeText(getApplicationContext(),
                        "接続できませんでした。設定アプリを開いてWiFiをONにしてください。",
                        Toast.LENGTH_LONG).show();
            } else {
                Toast.makeText(getApplicationContext(), "失敗:" +
                        status.getStatusMessage(), Toast.LENGTH_LONG).show();
            }
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == REQUEST_RESOLVE_ERROR) {
            mResolvingError = false;
            if (resultCode == Activity.RESULT_OK) {
                subscribe();

            } else if (resultCode == Activity.RESULT_CANCELED) {
                // User declined to opt-in. Reset application state here.
            } else {
                Toast.makeText(this, "Failed to resolve error with code " + resultCode,
                        Toast.LENGTH_LONG).show();
            }
        }

    }

    private void publish(String strMessage) {

        mMessage = new Message(strMessage.getBytes());

        if (!mGoogleApiClient.isConnected()) {
            if (!mGoogleApiClient.isConnecting()) {
                mGoogleApiClient.connect();
            }
        } else {
            PublishOptions options = new PublishOptions.Builder()
                    .setCallback(new PublishCallback() {
                        @Override
                        public void onExpired() {
                            // タイムアウト
                            Log.i(TAG, "onExpired");
                        }
                    }).build();

            Nearby.Messages.publish(mGoogleApiClient, mMessage, options)
                    .setResultCallback(new ResultCallback<Status>() {

                        @Override
                        public void onResult(Status status) {
                            if (status.isSuccess()) {
                                // 配信された時
                                Log.i(TAG, "配信完了");
                            } else {
                                // 配信されなかった時
                                Log.i(TAG, "配信失敗");

                                handleUnsuccessfulNearbyResult(status);
                            }
                        }
                    });
        }
    }

    private void unpublish() {
        if (mMessage != null) {
            Nearby.Messages.unpublish(mGoogleApiClient, mMessage);
        }
    }


}
